#!/usr/bin/python
# -*- coding: utf-8 -*-
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# @author : pascal.fautrero@ac-versailles.fr



import os
import math
import re
import tempfile
import sys
import shutil
import commands

# import PIL for windows and Linux
# For MAC OS X, use internal tool called "sips"
try:
    from PIL import Image
    HANDLE_PIL = True
except ImportError:
    if not sys.platform.startswith('darwin'):
        print "Requirement : Please, install python-pil package"
        sys.exit(1)
    else:
        HANDLE_PIL = False

import hashlib
import uuid
from xml.dom import minidom

from ctm import CurrentTransformation
import cubicsuperpath
from pikipiki import PageFormatter


class iaObject:
    """generate Image Active Object"""

    def __init__(self, console):
        """Init"""
        self.details = []
        self.scene = {}
        self.console = console

        # used to identify if background image must be resized for mobiles
        self.ratio = 1

        self.translation = 0
        self.jsonContent = ""
        self.filePath = ""

    def get_tag_value(self, node):
        if node.childNodes:
            return node.childNodes[0].nodeValue
        else:
            return ""

    def extractMetadatas(self, xml):

        metadatas = xml.getElementsByTagName('metadata')
        self.scene['date'] = ""
        self.scene['identifier'] = ""
        self.scene['coverage'] = ""
        self.scene['source'] = ""
        self.scene['relation'] = ""
        self.scene['creator'] = ""
        self.scene['rights'] = ""
        self.scene['publisher'] = ""
        self.scene['language'] = ""
        self.scene['keywords'] = ""
        self.scene['description'] = ""
        self.scene['contributor'] = ""
        if metadatas.item(0) is not None:

            metadata = metadatas.item(0).getElementsByTagName('dc:title')
            if metadata.item(0) is not None:
                if self.get_tag_value(metadata.item(0)) != "":
                    self.scene['title'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:date')
            if metadata.item(0) is not None:
                self.scene['date'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:coverage')
            if metadata.item(0) is not None:
                self.scene['coverage'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:relation')
            if metadata.item(0) is not None:
                self.scene['relation'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:source')
            if metadata.item(0) is not None:
                self.scene['source'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:identifier')
            if metadata.item(0) is not None:
                self.scene['identifier'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:creator')
            if metadata.item(0) is not None:
                metadata_value = metadata.item(0).getElementsByTagName('dc:title')
                if metadata_value.item(0) is not None:
                    self.scene['creator'] = self.get_tag_value(metadata_value.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:rights')
            if metadata.item(0) is not None:
                metadata_value = metadata.item(0).getElementsByTagName('dc:title')
                if metadata_value.item(0) is not None:
                    self.scene['rights'] = self.get_tag_value(metadata_value.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:publisher')
            if metadata.item(0) is not None:
                metadata_value = metadata.item(0).getElementsByTagName('dc:title')
                if metadata_value.item(0) is not None:
                    self.scene['publisher'] = self.get_tag_value(metadata_value.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:language')
            if metadata.item(0) is not None:
                self.scene['language'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:subject')
            if metadata.item(0) is not None:
                items = metadata.item(0).getElementsByTagName('rdf:li')
                for key_word in items:
                    if self.scene['keywords']:
                        self.scene['keywords'] += ","
                    self.scene['keywords'] += self.get_tag_value(key_word)

            metadata = metadatas.item(0).getElementsByTagName('dc:description')
            if metadata.item(0) is not None:
                self.scene['description'] = self.get_tag_value(metadata.item(0))

            metadata = metadatas.item(0).getElementsByTagName('dc:contributor')
            if metadata.item(0) is not None:
                metadata_value = metadata.item(0).getElementsByTagName('dc:title')
                if metadata_value.item(0) is not None:
                    self.scene['contributor'] = self.get_tag_value(metadata_value.item(0))

    def extractRaster(self, xlink):
        """ extract raster from xlink:href attribute """
        raster = xlink
        if not xlink.startswith("data:"):
            strStarter = 0
            if xlink.startswith("file://"):
                strStarter = len("file://")
            # look for windows drive letter
            if re.match("^[a-zA-Z]:.*$", xlink[strStarter + 1], re.UNICODE):
                strStarter = strStarter + 1
            # Embed image thanks to data URI Scheme
            imgName, imgExtension = os.path.splitext(xlink[strStarter:])
            imgMimeTypes = {}
            imgMimeTypes['.png'] = 'image/png'
            imgMimeTypes['.jpg'] = 'image/jpeg'
            imgMimeTypes['.jpeg'] = 'image/jpeg'
            imgMimeTypes['.gif'] = 'image/gif'
            imgMimeTypes['.bmp'] = 'image/bmp'
            rasterPref = u"data:" + \
                         imgMimeTypes[imgExtension.lower()] + u";base64,"
            if os.path.exists(xlink[strStarter:]):
                imgFile = xlink[strStarter:]
            else:
                localDir = os.path.dirname(self.filePath)
                imgFile = localDir + "/" + xlink[strStarter:]
            if os.path.exists(imgFile):
                with open(imgFile, 'rb') as img:
                    raster = rasterPref + img.read().encode("base64", "strict")
        return raster

    def analyzeSVG(self, filePath, maxNumPixels):
        """analyze svg file and fill self.details and self.scene"""
        self.details = []
        self.scene.clear()
        self.translation = 0
        self.ratio = 1

        # workaround to be able to read svg files 
        # generated with images actives 1
        # (xlink namespace is not available and must be added)

        with open(filePath, 'r') as svgfile:
            svgcontent = svgfile.read()
            if not re.search(r'xmlns:xlink=', svgcontent, re.M):
                svgcontent = svgcontent.replace("<svg", '<svg\nxmlns:xlink="http://www.w3.org/1999/xlink"')
                dirname = os.path.dirname(filePath)
                basename = os.path.basename(filePath)
                fixedfile = dirname + "/fixed_" + basename
                with open(fixedfile, 'w') as tempsvgfile:
                    tempsvgfile.write(svgcontent)
                filePath = fixedfile

        self.filePath = filePath

        self.xml = minidom.parse(filePath)

        head, tail = os.path.split(filePath)
        self.scene['intro_title'] = u"Description"
        self.scene['intro_detail'] = u"XIA - DANE Versailles"
        self.scene['image'] = ""
        self.scene['width'] = ""
        self.scene['height'] = ""
        self.scene['title'] = os.path.splitext(tail)[0]

        self.extractMetadatas(self.xml)

        # ==================== Look for background image

        images = self.xml.getElementsByTagName('image')
        if images.item(0) is not None:
            image = images.item(0)
            self.backgroundNode = image
            self.scene['width'] = image.attributes['width'].value
            self.scene['height'] = image.attributes['height'].value
            self.backgroundX = 0
            self.backgroundY = 0
            if image.hasAttribute('x'):
                self.backgroundX = float(image.attributes['x'].value)
            if image.hasAttribute('y'):
                self.backgroundY = float(image.attributes['y'].value)

            #print self.backgroundX
            #print self.backgroundY
            if (self.backgroundX != 0) or (self.backgroundY != 0):
                ctm = CurrentTransformation()
                ctm.extractTranslate([self.backgroundX * (-1), self.backgroundY * (-1)])
                self.translation = ctm.matrix

            #print self.translation

            desc = image.getElementsByTagName('desc')
            if desc.item(0) is None and image.parentNode.parentNode is not None:
                big_group = image.parentNode.parentNode
                #libreoffice svg export
                if big_group.nodeName == 'g':
                    if big_group.hasAttribute("class"):
                        if big_group.attributes["class"].value == "Graphic":
                            desc = big_group.getElementsByTagName('desc')
            if desc.item(0) is not None:
                #if desc.item(0).parentNode == image:
                self.scene['intro_detail'] = self.get_tag_value(desc.item(0))

            title = image.getElementsByTagName('title')
            if title.item(0) is None and image.parentNode.parentNode is not None:
                big_group = image.parentNode.parentNode
                #libreoffice svg export
                if big_group.nodeName == 'g':
                    if big_group.hasAttribute("class"):
                        if big_group.attributes["class"].value == "Graphic":
                            title = big_group.getElementsByTagName('title')
            if title.item(0) is not None:
                #if title.item(0).parentNode == image:
                self.scene['intro_title'] = self.get_tag_value(title.item(0))

            raster = self.extractRaster(image.attributes['xlink:href'].value)

            if image.hasAttribute("transform"):
                transformation = image.attributes['transform'].value
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                #print str(ctm.scaleX) + " " + str(ctm.scaleY)
                if ctm.scaleX != 1 or ctm.scaleY != 1:
                    self.scene['image'], self.scene['width'], self.scene['height'] = self.resizeImage(
                        raster,
                        int(float(self.scene['width'])) * ctm.scaleX,
                        int(float(self.scene['height'])) * ctm.scaleY)

            fixedRaster = self.fixRaster(raster, self.scene['width'], self.scene['height'])
            self.scene['image'] = fixedRaster

            # calculate ratio to resize background image down to maxNumPixels

            bgNumPixels = float(int(float(self.scene['width'])) * int(float(self.scene['height'])))
            if bgNumPixels > maxNumPixels:
                self.ratio = math.sqrt(maxNumPixels / bgNumPixels)

            if self.ratio != 1:
                self.scene['image'], self.scene['width'], self.scene['height'] = self.resizeImage(self.scene['image'],
                                                                                                  self.scene['width'],
                                                                                                  self.scene['height'])

        svgElements = ['rect', 'circle', 'ellipse', 'line', 'polyline', 'polygon', 'path', 'image', 'g']
        mainSVG = self.xml.getElementsByTagName('svg')
        if mainSVG[0]:
            # if there is only one root group
            # just remove it
            nb_root_groups = 0
            nb_root_elements = 0
            last_group = None
            for childnode in mainSVG[0].childNodes:
                if childnode.nodeName == "g":
                    nb_root_groups += 1
                    nb_root_elements += 1
                    last_group = childnode
                elif childnode.nodeName in svgElements:
                    nb_root_elements += 1

            if (nb_root_groups == 1) and (nb_root_elements == 1):
                mainSVG[0] = last_group
            else:
                # look for libreoffice draw svg export
                if last_group is not None:
                    if last_group.hasAttribute("class"):
                        if last_group.attributes["class"].value == "SlideGroup":
                            group_childs = []
                            self.print_node(last_group, group_childs)
                            for childnode in group_childs:
                                if childnode.hasAttribute("class"):
                                    if childnode.attributes["class"].value == "Page":
                                        mainSVG[0] = childnode
                                        break

            for childnode in mainSVG[0].childNodes:
                if childnode.parentNode.nodeName == mainSVG[0].nodeName:
                    if childnode.nodeName in svgElements:
                        newrecord = getattr(self, 'extract_' + childnode.nodeName)(childnode, "")
                        if newrecord is not None:
                            self.details.append(newrecord)

    def extract_circle(self, circle, stackTransformations):
        """Analyze circle"""
        record_circle = {}
        record_circle['id'] = hashlib.md5(str(uuid.uuid1())).hexdigest()
        if circle.hasAttribute("id"):
            record_circle['id'] = circle.attributes['id'].value

        record_circle['detail'] = self.getText("desc", circle)
        record_circle['title'] = self.getText("title", circle)
        record_circle['cx'] = unicode(0)
        record_circle['cy'] = unicode(0)
        record_circle['r'] = unicode(0)
        record_circle['options'] = ""

        if circle.hasAttribute("cx"):
            record_circle['cx'] = circle.attributes['cx'].value
            if self.translation != 0:
                record_circle['cx'] = float(record_circle['cx'])
        if circle.hasAttribute("cy"):
            record_circle['cy'] = circle.attributes['cy'].value
            if self.translation != 0:
                record_circle['cy'] = float(record_circle['cy'])
        if circle.hasAttribute("r"):
            record_circle['r'] = circle.attributes['r'].value

        if circle.hasAttribute("onclick"):
            str_onclick = circle.attributes['onclick'].value
            if str_onclick == "off":
                record_circle['options'] += " disable-click "
            else:
                record_circle['options'] += " " + str_onclick + " "
        if circle.hasAttribute("onmouseover"):
            str_onmouseover = circle.attributes['onmouseover'].value
            record_circle['options'] += " " + str_onmouseover + " "

        if record_circle['title'].startswith("http://") or \
                record_circle['title'].startswith("https://") or \
                record_circle['title'].startswith("//") or \
                record_circle['title'].startswith("./") or \
                record_circle['title'].startswith("../"):
            record_circle['options'] += " direct-link "

        if circle.hasAttribute("images_actives:zoomable"):
            str_zoom = circle.attributes['images_actives:zoomable'].value
            if str_zoom == "false":
                record_circle['fill'] = "#000000"


        if circle.hasAttribute("style"):
            str_style = circle.attributes['style'].value
            style = {}
            for item in str_style.split(";"):
                key, value = item.split(":")
                style[key] = value
            if 'fill' in style:
                record_circle['fill'] = style['fill']
            if 'stroke' in style:
                record_circle['stroke'] = style['stroke']
            if 'stroke-width' in style:
                record_circle['strokewidth'] = style['stroke-width']

        # ObjectToPath
        ctm = CurrentTransformation()
        record_circle['path'] = ctm.circleToPath(record_circle)

        p = cubicsuperpath.parsePath(record_circle['path'])
        record_circle['path'] = cubicsuperpath.formatPath(p)
        record_circle['x'] = unicode(0)
        record_circle['y'] = unicode(0)
        if stackTransformations == "":
            if circle.hasAttribute("transform"):
                transformation = circle.attributes['transform'].value
                ctm.analyze(transformation)

                ctm.applyTransformToPath(ctm.matrix, p)
                record_circle['path'] = cubicsuperpath.formatPath(p)

        # apply group transformation on current object
        else:
            transformations = stackTransformations.split("#")
            for transformation in transformations[::-1]:
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                ctm.applyTransformToPath(ctm.matrix, p)
                record_circle['path'] = cubicsuperpath.formatPath(p)

                #if ctm_group:
        #    ctm_group.applyTransformToPath(ctm_group.matrix,p)
        #    record_circle['path'] = cubicsuperpath.formatPath(p)

        if self.translation != 0:
            ctm = CurrentTransformation()
            ctm.applyTransformToPath(self.translation, p)
            record_circle['path'] = cubicsuperpath.formatPath(p)

        if self.ratio != 1:
            ctm = CurrentTransformation()
            ctm.extractScale([self.ratio])
            ctm.applyTransformToPath(ctm.matrix, p)
            record_circle['path'] = cubicsuperpath.formatPath(p)

        minX = 10000
        minY = 10000
        maxX = -10000
        maxY = -10000
        for cmd, params in cubicsuperpath.unCubicSuperPath(p):
            i = 0
            for p in params:
                if (i % 2 == 0):
                    if float(p) < float(minX):
                        minX = float(p)
                    if float(p) > float(maxX):
                        maxX = float(p)
                else:
                    if float(p) < float(minY):
                        minY = float(p)
                    if float(p) > float(maxY):
                        maxY = float(p)
                i = i + 1
        record_circle["minX"] = unicode(minX)
        record_circle["minY"] = unicode(minY)
        record_circle["maxX"] = unicode(maxX)
        record_circle["maxY"] = unicode(maxY)

        record_circle['path'] = '"' + record_circle['path'] + ' z"'
        return record_circle

    def extract_ellipse(self, ellipse, stackTransformations):
        """Analyze ellipse"""

        record_ellipse = {}
        record_ellipse['id'] = hashlib.md5(str(uuid.uuid1())).hexdigest()
        if ellipse.hasAttribute("id"):
            record_ellipse['id'] = ellipse.attributes['id'].value

        record_ellipse['detail'] = self.getText("desc", ellipse)
        record_ellipse['title'] = self.getText("title", ellipse)
        record_ellipse['cx'] = unicode(0)
        record_ellipse['cy'] = unicode(0)
        record_ellipse['rx'] = unicode(0)
        record_ellipse['ry'] = unicode(0)
        record_ellipse['options'] = ""

        if ellipse.hasAttribute("cx"):
            record_ellipse['cx'] = ellipse.attributes['cx'].value
            if self.translation != 0:
                record_ellipse['cx'] = float(record_ellipse['cx'])
        if ellipse.hasAttribute("cy"):
            record_ellipse['cy'] = ellipse.attributes['cy'].value
            if self.translation != 0:
                record_ellipse['cy'] = float(record_ellipse['cy'])
        if ellipse.hasAttribute("rx"):
            record_ellipse['rx'] = ellipse.attributes['rx'].value
        if ellipse.hasAttribute("ry"):
            record_ellipse['ry'] = ellipse.attributes['ry'].value

        if ellipse.hasAttribute("onclick"):
            str_onclick = ellipse.attributes['onclick'].value
            if str_onclick == "off":
                record_ellipse['options'] += " disable-click "
            else:
                record_ellipse['options'] += " " + str_onclick + " "
        if ellipse.hasAttribute("onmouseover"):
            str_onmouseover = ellipse.attributes['onmouseover'].value
            record_ellipse['options'] += " " + str_onmouseover + " "

        if record_ellipse['title'].startswith("http://") or \
                record_ellipse['title'].startswith("https://") or \
                record_ellipse['title'].startswith("//") or \
                record_ellipse['title'].startswith("./") or \
                record_ellipse['title'].startswith("../"):
            record_ellipse['options'] += " direct-link "

        if ellipse.hasAttribute("images_actives:zoomable"):
            str_zoom = ellipse.attributes['images_actives:zoomable'].value
            if str_zoom == "false":
                record_ellipse['fill'] = "#000000"

        if ellipse.hasAttribute("style"):
            str_style = ellipse.attributes['style'].value
            style = {}
            for item in str_style.split(";"):
                key, value = item.split(":")
                style[key] = value
            if 'fill' in style:
                record_ellipse['fill'] = style['fill']
            if 'stroke' in style:
                record_ellipse['stroke'] = style['stroke']
            if 'stroke-width' in style:
                record_ellipse['strokewidth'] = style['stroke-width']

        # ObjectToPath
        ctm = CurrentTransformation()
        record_ellipse['path'] = ctm.ellipseToPath(record_ellipse)

        p = cubicsuperpath.parsePath(record_ellipse['path'])
        record_ellipse['path'] = cubicsuperpath.formatPath(p)
        record_ellipse['x'] = unicode(0)
        record_ellipse['y'] = unicode(0)
        if stackTransformations == "":
            if ellipse.hasAttribute("transform"):
                transformation = ellipse.attributes['transform'].value
                ctm.analyze(transformation)

                ctm.applyTransformToPath(ctm.matrix, p)
                record_ellipse['path'] = cubicsuperpath.formatPath(p)

        # apply group transformation on current object
        else:
            transformations = stackTransformations.split("#")
            for transformation in transformations[::-1]:
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                ctm.applyTransformToPath(ctm.matrix, p)
                record_ellipse['path'] = cubicsuperpath.formatPath(p)

        if self.translation != 0:
            ctm = CurrentTransformation()
            ctm.applyTransformToPath(self.translation, p)
            record_ellipse['path'] = cubicsuperpath.formatPath(p)

        if self.ratio != 1:
            ctm = CurrentTransformation()
            ctm.extractScale([self.ratio])
            ctm.applyTransformToPath(ctm.matrix, p)
            record_ellipse['path'] = cubicsuperpath.formatPath(p)

        minX = 10000
        minY = 10000
        maxX = -10000
        maxY = -10000
        for cmd, params in cubicsuperpath.unCubicSuperPath(p):
            i = 0
            for p in params:
                if (i % 2 == 0):
                    if float(p) < float(minX):
                        minX = float(p)
                    if float(p) > float(maxX):
                        maxX = float(p)
                else:
                    if float(p) < float(minY):
                        minY = float(p)
                    if float(p) > float(maxY):
                        maxY = float(p)
                i = i + 1
        record_ellipse["minX"] = unicode(minX)
        record_ellipse["minY"] = unicode(minY)
        record_ellipse["maxX"] = unicode(maxX)
        record_ellipse["maxY"] = unicode(maxY)

        record_ellipse['path'] = '"' + record_ellipse['path'] + ' z"'
        return record_ellipse

    def extract_line(self, image, stackTransformations):
        """not yet implemented"""
        self.console.display("line is not implemented. Convert it to path.")

    def extract_polyline(self, image, stackTransformations):
        """not yet implemented"""
        self.console.display("polyline is not implemented. Convert it to path.")

    def extract_polygon(self, image, stackTransformations):
        """not yet implemented"""
        self.console.display("polygon is not implemented. Convert it to path.")

    def extract_image(self, image, stackTransformations):
        """Analyze images"""

        if not image.isSameNode(self.backgroundNode):
            record_image = {}
            record_image['id'] = hashlib.md5(str(uuid.uuid1())).hexdigest()
            if image.hasAttribute('id'):
                record_image['id'] = image.attributes['id'].value
            record_image['width'] = image.attributes['width'].value
            record_image['height'] = image.attributes['height'].value
            raster = self.extractRaster(image.attributes['xlink:href'].value)
            #fixedRaster = self.fixRaster(raster, record_image["width"], record_image['height'])
            record_image['image'] = raster
            record_image['detail'] = self.getText("desc", image)
            record_image['title'] = self.getText("title", image)
            record_image['x'] = unicode(0)
            record_image['y'] = unicode(0)
            record_image['options'] = ""

            if image.hasAttribute("x"):
                record_image['x'] = unicode((float(image.attributes['x'].value) - self.backgroundX) * self.ratio)
            if image.hasAttribute("y"):
                record_image['y'] = unicode((float(image.attributes['y'].value) - self.backgroundY) * self.ratio)

            if self.ratio != 1:
                record_image['image'], \
                record_image['width'], \
                record_image['height'] = \
                    self.resizeImage(record_image['image'], record_image['width'], record_image['height'])
            #if not sys.platform.startswith("darwin"):
            if HANDLE_PIL:
                record_image['image'], \
                record_image['width'], \
                record_image['height'], \
                newx, newy = self.cropImage(record_image['image'], record_image['width'], record_image['height'])
                record_image['y'] = unicode(int(float(record_image['y']) + float(newy)))
                record_image['x'] = unicode(int(float(record_image['x']) + float(newx)))

            if record_image['title'].startswith("http://") or \
                    record_image['title'].startswith("https://") or \
                    record_image['title'].startswith("//") or \
                    record_image['title'].startswith("./") or \
                    record_image['title'].startswith("../"):
                record_image['options'] += " direct-link "

            if image.hasAttribute("images_actives:zoomable"):
                str_zoom = image.attributes['images_actives:zoomable'].value
                if str_zoom == "false":
                    record_image['fill'] = "#000000"

            if image.hasAttribute("style"):
                str_style = image.attributes['style'].value
                style = {}
                for item in str_style.split(";"):
                    key, value = item.split(":")
                    style[key] = value
                if 'fill' in style:
                    record_image['fill'] = style['fill']
                if 'stroke' in style:
                    record_image['stroke'] = style['stroke']
                if 'stroke-width' in style:
                    record_image['strokewidth'] = style['stroke-width']

            if image.hasAttribute("onclick"):
                str_onclick = image.attributes['onclick'].value
                if str_onclick == "off":
                    record_image['options'] += " disable-click "
                else:
                    record_image['options'] += " " + str_onclick + " "

            if image.hasAttribute("onmouseover"):
                str_onmouseover = image.attributes['onmouseover'].value
                record_image['options'] += " " + str_onmouseover + " "

            if image.hasAttribute("transform"):
                transformation = image.attributes['transform'].value
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                record_image['x'] = ctm.translateX
                record_image['y'] = ctm.translateY

            minX = 10000
            minY = 10000
            maxX = -10000
            maxY = -10000
            if float(record_image['x']) < float(minX):
                minX = float(record_image['x'])
            if (float(record_image['x']) + float(record_image['width'])) > float(maxX):
                maxX = float(record_image['x']) + float(record_image['width'])
            if float(record_image['y']) < float(minY):
                minY = float(record_image['y'])
            if (float(record_image['y']) + float(record_image['height'])) > float(maxY):
                maxY = float(record_image['y']) + float(record_image['height'])

            record_image["minX"] = unicode(minX)
            record_image["minY"] = unicode(minY)
            record_image["maxX"] = unicode(maxX)
            record_image["maxY"] = unicode(maxY)

            return record_image

    def extract_rect(self, rect, stackTransformations):
        """Analyze rectangles"""

        record_rect = {}
        record_rect['id'] = hashlib.md5(str(uuid.uuid1())).hexdigest()
        if rect.hasAttribute("id"):
            record_rect['id'] = rect.attributes['id'].value
        record_rect['width'] = rect.attributes['width'].value
        record_rect['height'] = rect.attributes['height'].value

        # if dimensions are invalid, exclude this rectangle
        if not record_rect['width'] or not record_rect['height']:
            return

        record_rect['detail'] = self.getText("desc", rect)
        record_rect['title'] = self.getText("title", rect)
        record_rect['x'] = unicode(0)
        record_rect['y'] = unicode(0)
        record_rect['rx'] = unicode(0)
        record_rect['ry'] = unicode(0)
        record_rect['options'] = ""

        if rect.hasAttribute("x"):
            record_rect['x'] = rect.attributes['x'].value
            if self.translation != 0:
                record_rect['x'] = float(record_rect['x'])
        if rect.hasAttribute("y"):
            record_rect['y'] = rect.attributes['y'].value
            if self.translation != 0:
                record_rect['y'] = float(record_rect['y'])
        if rect.hasAttribute("rx"):
            record_rect['rx'] = rect.attributes['rx'].value
        if rect.hasAttribute("ry"):
            record_rect['ry'] = rect.attributes['ry'].value

        if rect.hasAttribute("onclick"):
            str_onclick = rect.attributes['onclick'].value
            if str_onclick == "off":
                record_rect['options'] += " disable-click "
            else:
                record_rect['options'] += " " + str_onclick + " "
        if rect.hasAttribute("onmouseover"):
            str_onmouseover = rect.attributes['onmouseover'].value
            record_rect['options'] += " " + str_onmouseover + " "

        if record_rect['title'].startswith("http://") or \
                record_rect['title'].startswith("https://") or \
                record_rect['title'].startswith("//") or \
                record_rect['title'].startswith("./") or \
                record_rect['title'].startswith("../"):
            record_rect['options'] += " direct-link "

        if rect.hasAttribute("images_actives:zoomable"):
            str_zoom = rect.attributes['images_actives:zoomable'].value
            if str_zoom == "false":
                record_rect['fill'] = "#000000"


        if rect.hasAttribute("style"):
            str_style = rect.attributes['style'].value
            style = {}
            for item in str_style.split(";"):
                key, value = item.split(":")
                style[key] = value
            if 'fill' in style:
                record_rect['fill'] = style['fill']
            if 'stroke' in style:
                record_rect['stroke'] = style['stroke']
            if 'stroke-width' in style:
                record_rect['strokewidth'] = style['stroke-width']

        # ObjectToPath                    
        ctm = CurrentTransformation()
        record_rect['path'] = ctm.rectToPath(record_rect)

        p = cubicsuperpath.parsePath(record_rect['path'])
        record_rect['path'] = cubicsuperpath.formatPath(p)
        record_rect['x'] = unicode(0)
        record_rect['y'] = unicode(0)
        if stackTransformations == "":
            if rect.hasAttribute("transform"):
                transformation = rect.attributes['transform'].value
                ctm.analyze(transformation)

                ctm.applyTransformToPath(ctm.matrix, p)
                record_rect['path'] = cubicsuperpath.formatPath(p)

        # apply group transformation on current object
        else:
            transformations = stackTransformations.split("#")
            for transformation in transformations[::-1]:
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                ctm.applyTransformToPath(ctm.matrix, p)
                record_rect['path'] = cubicsuperpath.formatPath(p)

                #if ctm_group:
        #    ctm_group.applyTransformToPath(ctm_group.matrix,p)
        #    record_rect['path'] = cubicsuperpath.formatPath(p)

        if self.translation != 0:
            ctm = CurrentTransformation()
            ctm.applyTransformToPath(self.translation, p)
            record_rect['path'] = cubicsuperpath.formatPath(p)

        if self.ratio != 1:
            ctm = CurrentTransformation()
            ctm.extractScale([self.ratio])
            ctm.applyTransformToPath(ctm.matrix, p)
            record_rect['path'] = cubicsuperpath.formatPath(p)

        minX = 10000
        minY = 10000
        maxX = -10000
        maxY = -10000
        for cmd, params in cubicsuperpath.unCubicSuperPath(p):
            i = 0
            for p in params:
                if (i % 2 == 0):
                    if float(p) < float(minX):
                        minX = float(p)
                    if float(p) > float(maxX):
                        maxX = float(p)
                else:
                    if float(p) < float(minY):
                        minY = float(p)
                    if float(p) > float(maxY):
                        maxY = float(p)
                i = i + 1
        record_rect["minX"] = unicode(minX)
        record_rect["minY"] = unicode(minY)
        record_rect["maxX"] = unicode(maxX)
        record_rect["maxY"] = unicode(maxY)

        record_rect['path'] = '"' + record_rect['path'] + ' z"'
        return record_rect

    def extract_path(self, path, stackTransformations):
        """Analyze paths"""

        record = {}
        record["path"] = ""
        record["fill"] = ""
        record["id"] = hashlib.md5(str(uuid.uuid1())).hexdigest()
        if path.hasAttribute("id"):
            record["id"] = path.attributes['id'].value
        record["path"] = path.attributes['d'].value. \
            replace("&#xd;&#xa;", " "). \
            replace("&#x9;", " "). \
            replace("\n", " "). \
            replace("\t", " "). \
            replace("\r", " ")

        # if path is invalid, exclude this detail
        if not record["path"]:
            return

        record["style"] = ""
        record['detail'] = self.getText("desc", path)
        record['title'] = self.getText("title", path)
        record['x'] = unicode(0)
        record['y'] = unicode(0)
        record['options'] = ""

        if path.hasAttribute("onclick"):
            str_onclick = path.attributes['onclick'].value
            if str_onclick == "off":
                record['options'] += " disable-click "
            else:
                record['options'] += " " + str_onclick + " "

        if path.hasAttribute("onmouseover"):
            str_onmouseover = path.attributes['onmouseover'].value
            record['options'] += " " + str_onmouseover + " "

        if record['title'].startswith("http://") or \
                record['title'].startswith("https://") or \
                record['title'].startswith("//") or \
                record['title'].startswith("./") or \
                record['title'].startswith("../"):
            record['options'] += " direct-link "

        if path.hasAttribute("images_actives:zoomable"):
            str_zoom = path.attributes['images_actives:zoomable'].value
            if str_zoom == "false":
                record['fill'] = "#000000"


        if path.hasAttribute("style") and (path.attributes['style'].value != ""):
            str_style = path.attributes['style'].value
            style = {}
            for item in str_style.split(";"):
                key, value = item.split(":")
                style[key] = value
            if 'fill' in style:
                record["fill"] = style['fill']
            if 'stroke' in style:
                record['stroke'] = style['stroke']
            if 'stroke-width' in style:
                record['strokewidth'] = style['stroke-width']

        if path.hasAttribute("x"):
            record['x'] = path.attributes['x'].value
        if path.hasAttribute("y"):
            record['y'] = path.attributes['y'].value

        # ObjectToPath
        p = cubicsuperpath.parsePath(record['path'])
        record['path'] = cubicsuperpath.formatPath(p)

        if stackTransformations == "":
            if path.hasAttribute("transform"):
                transformation = path.attributes['transform'].value
                ctm = CurrentTransformation()
                ctm.analyze(transformation)

                ctm.applyTransformToPath(ctm.matrix, p)
                record['path'] = cubicsuperpath.formatPath(p)

        # apply group transformation on current object
        # if ctm_group:
        #    ctm_group.applyTransformToPath(ctm_group.matrix,p)
        #    record['path'] = cubicsuperpath.formatPath(p)

        else:
            transformations = stackTransformations.split("#")
            for transformation in transformations[::-1]:
                ctm = CurrentTransformation()
                ctm.analyze(transformation)
                ctm.applyTransformToPath(ctm.matrix, p)
                record['path'] = cubicsuperpath.formatPath(p)

        if self.translation != 0:
            #print("apply translation")
            ctm = CurrentTransformation()
            ctm.applyTransformToPath(self.translation, p)
            record['path'] = cubicsuperpath.formatPath(p)

        if self.ratio != 1:
            ctm = CurrentTransformation()
            ctm.extractScale([self.ratio])
            ctm.applyTransformToPath(ctm.matrix, p)
            record['path'] = cubicsuperpath.formatPath(p)

        if record["path"].lower().find("z") == -1:
            record["path"] += " z"
        record['path'] = '"' + record['path'] + '"'
        minX = 10000
        minY = 10000
        maxX = -10000
        maxY = -10000
        for cmd, params in cubicsuperpath.unCubicSuperPath(p):
            i = 0
            for p in params:
                if (i % 2 == 0):
                    if float(p) < float(minX):
                        minX = float(p)
                    if float(p) > float(maxX):
                        maxX = float(p)
                else:
                    if float(p) < float(minY):
                        minY = float(p)
                    if float(p) > float(maxY):
                        maxY = float(p)
                i = i + 1
        record["minX"] = unicode(minX)
        record["minY"] = unicode(minY)
        record["maxX"] = unicode(maxX)
        record["maxY"] = unicode(maxY)
        if record["path"]:
            return record

    def getText(self, type, element):
        """ type can be 'desc' or 'title' """
        text = element.getElementsByTagName(type)
        if text.item(0) is not None:
            if text.item(0).parentNode == element:
                return self.get_tag_value(text.item(0))
        return ""

    def print_node(self, root, childs):
        if root.childNodes:
            for node in root.childNodes:
                if node.nodeType == node.ELEMENT_NODE:
                    childs.append(node)
                    self.print_node(node, childs)

    def linearize_childs(self, root, childs, stackTransform):

        if root.hasAttribute("transform"):
            stackTransform.append(root.attributes['transform'].value)
        entry = {}
        entry["node"] = root
        entry["transform"] = "#".join(stackTransform)
        childs.append(entry)

        if root.childNodes:
            for node in root.childNodes:
                if node.nodeType == node.ELEMENT_NODE:
                    #childs.append(node)
                    self.linearize_childs(node, childs, stackTransform)

        if root.hasAttribute("transform"):
            stackTransform.pop()

    def extract_g(self, group, stackTransformations):
        """Analyze a svg group"""

        record = {}
        record["id"] = hashlib.md5(str(uuid.uuid1())).hexdigest()
        if group.hasAttribute("id"):
            record["id"] = group.attributes['id'].value
        record['title'] = self.getText("title", group)
        record['detail'] = self.getText("desc", group)
        record["group"] = []
        record["options"] = ""

        minX = 10000
        minY = 10000
        maxX = 0
        maxY = 0

        if group.hasAttribute("onclick"):
            str_onclick = group.attributes['onclick'].value
            if str_onclick == "off":
                record['options'] += " disable-click "
            else:
                record['options'] += " " + str_onclick + " "

        svgElements = ['rect', 'circle', 'ellipse', 'line', 'polyline', 'polygon', 'path', 'image']
        group_childs = []
        stackTransform = []
        self.linearize_childs(group, group_childs, stackTransform)

        for childentry in group_childs:
            childnode = childentry['node']
            #print childentry['node']
            #print childentry['transform']
            if childnode.nodeName in svgElements:
                newrecord = getattr(self, 'extract_' + \
                                    childnode.nodeName)(childnode, childentry['transform'])
                if newrecord is not None:
                    #newrecord["options"] += record["options"]
                    #record["options"] =  newrecord["options"]
                    record["group"].append(newrecord)

                    if record["detail"] == "":
                        record['detail'] = newrecord['detail']
                    if record["title"] == "":
                        record['title'] = newrecord['title']

                    if float(newrecord["minX"]) < minX:
                        minX = float(newrecord["minX"])
                    if float(newrecord["minY"]) < minY:
                        minY = float(newrecord["minY"])
                    if float(newrecord["maxX"]) > maxX:
                        maxX = float(newrecord["maxX"])
                    if float(newrecord["maxY"]) > maxY:
                        maxY = float(newrecord["maxY"])

        # look for title and description in subgroups if not yet available
        if (record['title'] == "") or (record['detail'] == ""):
            subgroups = group.getElementsByTagName('g')
            for subgroup in subgroups:
                if record["detail"] == "":
                    record['detail'] = self.getText("desc", subgroup)
                if record["title"] == "":
                    record['title'] = self.getText("title", subgroup)

        if record['title'] == "":
            if group.hasAttribute("inkscape:label"):
                record['title'] = group.attributes["inkscape:label"].value

        record["minX"] = unicode(minX)
        record["minY"] = unicode(minY)
        record["maxX"] = unicode(maxX)
        record["maxY"] = unicode(maxY)

        if record["group"]:
            return record

    def fixRaster(self, raster, rasterWidth, rasterHeight):
        dirname = tempfile.mkdtemp()
        newraster = raster

        rasterStartPosition = raster.find('base64,') + 7
        rasterEncoded = raster[rasterStartPosition:]
        rasterPrefix = raster[0:rasterStartPosition]
        extension = re.search('image/(.*);base64', rasterPrefix)
        if extension is not None:
            if extension.group(1):
                imageFile = dirname + os.path.sep + "image." + extension.group(1)
                imageFileFixed = dirname + \
                                 os.path.sep + "image_small." + extension.group(1)
                with open(imageFile, "wb") as bgImage:
                    bgImage.write(rasterEncoded.decode("base64"))

                #if sys.platform.startswith('darwin'):
                if not HANDLE_PIL:
                    shutil.copyfile(imageFile, imageFileFixed)
                    w = commands.getstatusoutput('sips -g pixelWidth {0}'.format(imageFile))
                    if w != rasterWidth:
                        commands.getstatusoutput('sips -z {0} {1} {2}'.format(rasterHeight,
                                                                              rasterWidth,
                                                                              imageFileFixed))

                        with open(imageFileFixed, 'rb') as fixedImage:
                            rasterFixedEncoded = fixedImage.read().encode("base64")
                            newraster = rasterPrefix + rasterFixedEncoded

                else:
                    currentImg = Image.open(imageFile)
                    (w, h) = currentImg.size
                    if w != rasterWidth:
                        newwidth = int(float(rasterWidth))
                        newheight = int(float(rasterHeight))
                        resizedImg = currentImg.resize((newwidth, newheight), Image.BICUBIC)
                        resizedImg.save(imageFileFixed)

                        with open(imageFileFixed, 'rb') as fixedImage:
                            rasterFixedEncoded = fixedImage.read().encode("base64")
                            newraster = rasterPrefix + rasterFixedEncoded

        else:
            self.console.display('ERROR : fixRaster() - image is not embedded')
        shutil.rmtree(dirname)
        return newraster


    def cropImage(self, raster, rasterWidth, rasterHeight):
        dirname = tempfile.mkdtemp()
        newraster = raster
        newrasterWidth = rasterWidth
        newrasterHeight = rasterHeight
        x_delta = 0
        y_delta = 0
        rasterStartPosition = raster.find('base64,') + 7
        rasterEncoded = raster[rasterStartPosition:]
        rasterPrefix = raster[0:rasterStartPosition]
        extension = re.search('image/(.*);base64', rasterPrefix)
        if extension is not None:
            if extension.group(1):
                imageFile = dirname + os.path.sep + "image." + extension.group(1)
                imageFileSmall = dirname + \
                                 os.path.sep + "image_small." + extension.group(1)
                with open(imageFile, "wb") as bgImage:
                    bgImage.write(rasterEncoded.decode("base64"))

                #if not sys.platform.startswith('darwin'):
                if HANDLE_PIL:
                    im = Image.open(imageFile, 'r')
                    im = im.convert("RGBA")
                    pix_val = list(im.getdata())
                    (w, h) = im.size
                    alpha_threshold = 100
                    y_delta = 0
                    stop_scan = 0
                    for y in range(h):
                        row = y * w
                        for x in range(w):
                            transparency = pix_val[x + row][3] - alpha_threshold
                            if transparency >= 0:
                                stop_scan = 1
                                break
                        if stop_scan:
                            break
                        else:
                            y_delta += 1

                    y_delta2 = 0
                    stop_scan = 0
                    for y in range(h - 1, 0, -1):
                        row = y * w
                        for x in range(w - 1, 0, -1):
                            transparency = pix_val[x + row][3] - alpha_threshold
                            if transparency >= 0:
                                stop_scan = 1
                                break
                        if stop_scan:
                            break
                        else:
                            y_delta2 += 1

                    x_delta = 0
                    stop_scan = 0
                    for x in range(0, w - 1):
                        for y in range(0, h - 1):
                            row = y * w
                            transparency = pix_val[x + row][3] - alpha_threshold
                            if transparency >= 0:
                                stop_scan = 1
                                break
                        if stop_scan:
                            break
                        else:
                            x_delta += 1

                    x_delta2 = 0
                    stop_scan = 0
                    for x in range(w - 1, 0, -1):
                        for y in range(h - 1, 0, -1):
                            row = y * w
                            transparency = pix_val[x + row][3] - alpha_threshold
                            if transparency >= 0:
                                stop_scan = 1
                                break
                        if stop_scan:
                            break
                        else:
                            x_delta2 += 1

                    croppedBg = im.crop((x_delta, y_delta, w - x_delta2, h - y_delta2))
                    croppedBg.save(imageFileSmall)

                    with open(imageFileSmall, 'rb') as bgSmallImage:
                        rasterSmallEncoded = bgSmallImage.read(). \
                            encode("base64")
                        newraster = rasterPrefix + \
                                    rasterSmallEncoded
                    newrasterHeight = int((h - y_delta2 - y_delta) * float(rasterHeight) / h)
                    newrasterWidth = int((w - x_delta - x_delta2) * float(rasterWidth) / w)
        else:
            self.console.display('ERROR : cropImage() - image is not embedded ' + raster)
        shutil.rmtree(dirname)
        return [newraster, unicode(newrasterWidth), unicode(newrasterHeight), x_delta, y_delta]

    def resizeImage(self, raster, rasterWidth, rasterHeight):
        """ 
        if needed, we must resize background image to be usable
        on tablets and mobiles. iPad limitation is approximatly 5Mb.
        """

        dirname = tempfile.mkdtemp()
        newraster = raster
        newrasterWidth = rasterWidth
        newrasterHeight = rasterHeight
        rasterStartPosition = raster.find('base64,') + 7
        rasterEncoded = raster[rasterStartPosition:]
        rasterPrefix = raster[0:rasterStartPosition]
        extension = re.search('image/(.*);base64', rasterPrefix)
        if extension is not None:
            if extension.group(1):
                imageFile = dirname + os.path.sep + "image." + extension.group(1)
                imageFileSmall = dirname + \
                                 os.path.sep + "image_small." + extension.group(1)
                with open(imageFile, "wb") as bgImage:
                    bgImage.write(rasterEncoded.decode("base64"))
                if self.ratio != 1:
                    # Background image is too big to be used on mobiles
                    #if sys.platform.startswith('darwin'):
                    if not HANDLE_PIL:
                        oldwidth = int(float(rasterWidth))
                        oldheight = int(float(rasterHeight))
                        newwidth = int(oldwidth * self.ratio)
                        newheight = int(oldheight * self.ratio)
                        shutil.copyfile(imageFile, imageFileSmall)
                        commands.getstatusoutput('sips -z {0} {1} {2}'.format(newheight, newwidth, imageFileSmall))

                        with open(imageFileSmall, 'rb') as bgSmallImage:
                            rasterSmallEncoded = bgSmallImage.read().encode("base64")
                            newraster = rasterPrefix + rasterSmallEncoded

                        newrasterWidth = newwidth
                        newrasterHeight = newheight

                    else:
                        # "Platform Linux or Windows : resizing using PIL"
                        currentBg = Image.open(imageFile)
                        oldwidth = int(float(rasterWidth))
                        oldheight = int(float(rasterHeight))
                        newwidth = int(oldwidth * self.ratio)
                        newheight = int(oldheight * self.ratio)
                        resizedBg = currentBg.resize((newwidth, newheight), Image.BICUBIC)
                        resizedBg.save(imageFileSmall)

                        with open(imageFileSmall, 'rb') as bgSmallImage:
                            rasterSmallEncoded = bgSmallImage.read().encode("base64")
                            newraster = rasterPrefix + rasterSmallEncoded

                        newrasterWidth = newwidth
                        newrasterHeight = newheight
        else:
            self.console.display('ERROR : image is not embedded')
        shutil.rmtree(dirname)
        return [newraster, unicode(newrasterWidth), unicode(newrasterHeight)]


    def generateJSON(self):
        """ generate json file"""
        final_str = u""
        final_str += u'var scene = {\n'
        for entry in self.scene:
            if entry == "image":
                final_str += u'"' + entry + u'":"' + \
                             self.scene[entry]. \
                                 replace("\n", ""). \
                                 replace("\t", ""). \
                                 replace("\r", "") + u'",\n'
            else:
                final_str += u'"' + entry + u'":"' + \
                             PageFormatter(self.scene[entry]).print_html(). \
                                 replace('"', "'"). \
                                 replace("\n", " "). \
                                 replace("\t", " "). \
                                 replace("\r", " ") + u'",\n'
        final_str += u'};\n'

        final_str += u'var details = [\n'
        for detail in self.details:
            final_str += u'{\n'
            for entry in detail:
                if entry == "group":
                    final_str += u'  "' + entry + u'": [\n'
                    for element in detail['group']:
                        final_str += '  {\n'
                        for entry2 in element:
                            if entry2 == "path":
                                final_str += u'  "' + entry2 + u'":' + \
                                             element[entry2]. \
                                                 replace('"', "'"). \
                                                 replace("\n", " "). \
                                                 replace("\t", " "). \
                                                 replace("\r", " ") + u',\n'
                            elif entry2 == "image" or entry2 == "title":
                                final_str += u'  "' + entry2 + u'":"' + \
                                             element[entry2]. \
                                                 replace('"', "'"). \
                                                 replace("\n", " "). \
                                                 replace("\t", " "). \
                                                 replace("\r", " ") + u'",\n'
                            else:
                                final_str += u'      "' + entry2 + u'":"' + \
                                             PageFormatter(element[entry2]).print_html(). \
                                                 replace('"', "'"). \
                                                 replace("\n", " "). \
                                                 replace("\t", " "). \
                                                 replace("\r", " ") + u'",\n'
                        final_str += u'  },\n'
                    final_str += u'  ],\n'
                elif entry == "path":
                    final_str += u'  "' + entry + u'":' + detail[entry] + ',\n'
                elif entry == "image":
                    final_str += u'  "' + entry + u'":"' + detail[entry]. \
                        replace('"', "'"). \
                        replace("\n", " "). \
                        replace("\t", " "). \
                        replace("\r", " ") + u'",\n'
                elif entry == "detail":
                    final_str += u'  "' + entry + u'":"' + \
                                 PageFormatter(detail[entry]).print_html(). \
                                     replace('"', "'"). \
                                     replace("\n", " "). \
                                     replace("\t", " "). \
                                     replace("\r", " ") + \
                                 u'",\n'
                else:
                    if type(detail[entry]) is not str and type(detail[entry]) is not unicode:
                        detail[entry] = unicode(detail[entry])
                    final_str += u'  "' + entry + u'":"' + \
                                 detail[entry]. \
                                     replace('"', "'"). \
                                     replace('\t', " "). \
                                     replace('\r', " "). \
                                     replace('\n', " ") + \
                                 u'",\n'
            final_str += u'},\n'
        final_str += u'];\n'
        self.jsonContent = final_str



#!/usr/bin/python
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# @author : pascal.fautrero@ac-versailles.fr

"""xia-converter.

Usage:
  xia.py [<input-file>]
  xia.py -i <input-file> -o <output-dir>  -t <theme>
  xia.py (-h | --help)
  xia.py --version

Options:
  -h --help     Show this screen.
  --version     Show version.

"""

try:
    import Tkinter, Tkconstants, tkFileDialog
except ImportError:
    import sys
    print "Requirement : Please, install python-tk package"
    sys.exit(1)

from xiaconverter.mainwindow import IADialog
import ConfigParser
from xiaconverter.docopt import docopt
from xiaconverter.xiaconsole import XIAConsole
from xiaconverter.loggerconsole import LoggerConsole

if __name__=='__main__':
    
    config = ConfigParser.ConfigParser()
    config.read("xia.cnf")
    numVersion = config.get('version', 'numVersion')
    releaseVersion = config.get('version', 'releaseVersion')
    imagesPath = config.get('paths', 'imagesPath')
    langPath = config.get('paths', 'langPath')
    fontsPath = config.get('paths', 'fontsPath')
    themesPath = config.get('paths', 'themesPath')
    labjsLib = config.get('paths', 'labjsLib')
    jqueryLib = config.get('paths', 'jqueryLib')
    kineticLib = config.get('paths', 'kineticLib')
    sha1Lib = config.get('paths', 'sha1Lib')

    arguments = docopt(__doc__)

    console = LoggerConsole()

    if arguments["--version"]:
        print(numVersion + releaseVersion)
    elif arguments["-i"] and arguments["-o"] and arguments["-t"]:
        input_file = arguments["<input-file>"]
        output_dir = arguments["<output-dir>"]
        selected_theme = arguments["<theme>"]
        xia = XIAConsole(langPath, themesPath, fontsPath, labjsLib, jqueryLib, kineticLib, sha1Lib, input_file,
                         output_dir, selected_theme, console)
        xia.createIA()
    else:
        filename = ""
        if arguments["<input-file>"] is not None:
            filename = arguments["<input-file>"]
        root = Tkinter.Tk()
        root.title("XIA " + numVersion + releaseVersion)
        root.geometry("465x310")
        root.resizable(0,0)
        img = Tkinter.PhotoImage(file=imagesPath + '/xia64.gif')
        root.tk.call('wm', 'iconphoto', root._w, img)    
        IADialog(root, console, langPath, imagesPath, themesPath, fontsPath, labjsLib, jqueryLib, kineticLib, sha1Lib,
                 filename).pack(side="left")
        root.mainloop()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# source code from pikipiki parser (sligthly modified)


import re
import string
import os
from datetime import datetime
import hashlib
import base64
import uuid
import math

class PageFormatter:
    """Object that turns Wiki markup into HTML.

    All formatting commands can be parsed one line at a time, though
    some state is carried over between lines.
    """
    def __init__(self, raw):
        if type(raw) == str:
            raw = raw.decode("utf-8")
        elif type(raw) != unicode:
            raw = unicode(raw)
        self.raw = raw
        self.is_em = self.is_b = 0
        self.list_indents = []
        self.in_pre = 0
        self.in_li = 0
        self.hidden_block = []
        self.final_str = ""

    def _emph_repl(self, word):
        if len(word) == 3:
            self.is_b = not self.is_b
            return [u'</b>', u'<b>'][self.is_b]
        else:
            self.is_em = not self.is_em
            return [u'</em>', u'<em>'][self.is_em]

    def _rule_repl(self, word):
        s = self._undent()
        if len(word) <= 4:
            s = s + u"\n<hr/>\n"
        else:
            s = s + u"\n<hr size=%d/>\n" % (len(word) - 2 )
        return s

    def _url_repl(self, word):
        return u'<a href="%s" target="_blank">%s</a>\n' % (word, word)

    def _videostart_repl(self, word):
        return u'<video controls preload="none" data-state="autostart">\n\t\
            <source type="video/mp4" src="%s.mp4" />\n\t\
            <source type="video/ogg" src="%s.ogv" />\n\t\
            <source type="video/webm" src="%s.webm" />\n\
            </video>\n' % (os.path.splitext(word)[0], \
              os.path.splitext(word)[0], os.path.splitext(word)[0])

    def _video_repl(self, word):
        return u'<video controls preload="none" data-state="none">\n\t\
            <source type="video/mp4" src="%s.mp4" />\n\t\
            <source type="video/ogg" src="%s.ogv" />\n\t\
            <source type="video/webm" src="%s.webm" />\n\
            </video>\n' % (os.path.splitext(word)[0], \
              os.path.splitext(word)[0], os.path.splitext(word)[0])

    def _img_repl(self, word):
        return u'<img src="%s"/>\n' % (word)

    def _iframe_repl(self, word):
        word_url = word.split("src=&quot;")[1].split("&quot;")[0]
        if word_url[0:2] == "//":
            word_url = "http:" + word_url
        iframe_width = 0
        iframe_height = 0
        if len(word.split('width="')) > 1:
            iframe_width = word.split('width="')[1].split('"')[0]
        if len(word.split('height="')) > 1:
            iframe_height = word.split('height="')[1].split('"')[0]
        videoClass = 'videoWrapper4_3'
        if iframe_width and iframe_height:
            ratio = (float(iframe_height) / float(iframe_width)) * 16
            if (ratio == 9):
                videoClass = 'videoWrapper16_9'
        return u'<div class="' + videoClass + \
          '" data-iframe="%s"></div>\n' % (word_url)

    def _iframe2_repl(self, word):
        
        word_url = ""
        srccheck = re.search('src=("|\')(.*?)("|\')', word, re.IGNORECASE|re.DOTALL)
        if srccheck:
            word_url = srccheck.group(2)        

        if word_url[0:2] == "//":
            word_url = "http:" + word_url
            
        iframe_width = ""
        widthcheck = re.search('width=("|\')(.*?)("|\')', word, re.IGNORECASE|re.DOTALL)
        if widthcheck:
            iframe_width = widthcheck.group(2)        
        
        iframe_height = ""
        heightcheck = re.search('height=("|\')(.*?)("|\')', word, re.IGNORECASE|re.DOTALL)
        if heightcheck:
            iframe_height = heightcheck.group(2)        

        videoClass = 'videoWrapper4_3'
        if iframe_width and iframe_height:
            ratio = (float(iframe_height) / float(iframe_width)) * 16
            if (ratio == 9):
                videoClass = 'videoWrapper16_9'
        return u'<div class="' + videoClass + \
          '" data-iframe="%s"></div>\n' % (word_url)
    
    def _audiostart_repl(self, word):
        return u'<audio controls data-state="autostart">\n\t\
            <source type="audio/ogg" src="%s.ogg" />\n\t\
            <source type="audio/mp3" src="%s.mp3" />\n\
            </audio>\n' % (os.path.splitext(word)[0], os.path.splitext(word)[0])


    def _audio_repl(self, word):
        return u'<audio controls data-state="none">\n\t\
            <source type="audio/ogg" src="%s.ogg" />\n\t\
            <source type="audio/mp3" src="%s.mp3" />\n\
            </audio>\n' % (os.path.splitext(word)[0], os.path.splitext(word)[0])

    def _email_repl(self, word):
        return u'<a href="mailto:%s">%s</a>\n' % (word, word)

    def _ent_repl(self, s):
        return {'&': '&amp;',
                '<': '&lt;',
                '>': '&gt;'}[s]

    def _li_repl(self, match):
        self.in_li = 1
        return u'<li>'

    def _link_repl(self, word):
        word_filtered = re.sub(' +', ' ', word[1:-1])
        word_displayed = ""
        word_url = ""
        for i in word_filtered.split(" "):
            if word_url == "":
                word_url = i
            elif word_displayed == "":
                word_displayed = i
            else:
                word_displayed = word_displayed + " " + i
        if word_displayed == "":
            word_displayed = word_url
        return u'<a href="%s" target="_blank">%s</a>' %(word_url, word_displayed)

    def _ialink_repl(self, word):
        """compatibility with image active 1 format"""
        subword = word.split("@")
        return u'<a href="%s" target="_blank">%s</a>' %(subword[1][:-1], subword[0][1:])

    def _pre_repl(self, word):
        if word == '{{{' and not self.in_pre:
            self.in_pre = 1
            return u'<pre>\n'
        elif self.in_pre:
            self.in_pre = 0
            return u'</pre>\n'
        else:
            return u''

    def _hidden_block_repl(self, word):
        if word.startswith('[['):
            #self.hidden_block.append(1)
            stack_value = 0
            data_password=""
            code_present = re.search('\(code=(.*)\)', word, re.IGNORECASE|re.DOTALL)
            content =  word[2:len(word) - 1]
            final_result = ""
            if code_present:
                password = code_present.group(1)
                stack_value = password
                data_password = 'data-password="' + hashlib.sha1(password.encode("utf-8")).hexdigest() + '"'
                content = re.sub('\(code=(.*)\)', '', content)
            
            random_id = hashlib.md5(str(uuid.uuid1())).hexdigest()
            final_result =  u'<div style="margin-top:5px;margin-bottom:5px;">' + \
                u'<a class="button" href="#" ' + data_password + \
                u' data-target="' + random_id + '">' + \
                content + \
                u'</a>' + \
                u'</div>'
            if data_password != "":
                final_result += u'<form class="unlock" style="display:none;" id="form_' + random_id + '">' + \
                            u'<input type="text">' + \
                            u'<input type="submit" data-target="' + random_id + '" value="" ' + data_password + '>' + \
                            u'</form>'
            final_result += u'<div class="response" id="response_' + random_id + '">'
            if data_password != "":
                final_result += u'<!-- ==HIDDEN_BLOCK== -->'
            
            self.hidden_block.append(stack_value)
            return final_result
        
        elif len(self.hidden_block):
            password = self.hidden_block.pop()
            current_str = word[0:word.find("]]")]
            if password:
                # encrypt hidden_block
                start_block = self.final_str.rfind("<!-- ==HIDDEN_BLOCK== -->")
                str_to_encrypt = self.final_str[start_block + len("<!-- ==HIDDEN_BLOCK== -->"):]
                str_to_encrypt = str_to_encrypt + current_str
                while len(password) < len(str_to_encrypt):
                    password += password
                str_encrypted = base64.b64encode(self.str_xor(str_to_encrypt.encode("utf-8"), password))
                self.final_str = self.final_str[0:start_block] + str_encrypted
                return u'</div>\n'
            else:
                return current_str + u'</div>\n'
        else:
            return u''

    def str_xor(self, s1, s2):
        return "".join([chr(ord(c1) ^ ord(c2)) for (c1,c2) in zip(s1,s2)])
        
    def _indent_level(self):
        return len(self.list_indents) and self.list_indents[-1]

    def _indent_to(self, new_level):
        s = u''
        while self._indent_level() > new_level:
            del(self.list_indents[-1])
            s = s + u'</ul>\n'
        while self._indent_level() < new_level:
            self.list_indents.append(new_level)
            s = s + u'<ul>\n'
        return s

    def _undent(self):
        res = u'</ul>' * len(self.list_indents)
        self.list_indents = []
        return res

    def replace(self, match):
        for type, hit in match.groupdict().items():
            if hit:
                if hasattr(self, '_' + type + '_repl'):
                    return getattr(self, '_' + type + '_repl')(hit,)
                else:
                    return "Can't handle match %s" % type

    def print_html(self):
        # For each line, we scan through looking for magic
        # strings, outputting verbatim any intervening text
        #final_str = u""
        scan_re = re.compile(
            r"(?:(?P<emph>\*{2,3})"
            + r'|(?P<iframe2><iframe(.*)></iframe>)'
            + r"|(?P<iframe>&lt;iframe(.*)&gt;&lt;/iframe&gt;)"        
            + r"|(?P<ent>[<>&])"
            + r"|(?P<rule>-{4,})"
            + r"|(?P<videostart>[^\s'\"]+\.(ogv|mp4|webm)(\s*)autostart$)"
            + r"|(?P<video>[^\s'\"]+\.(ogv|mp4|webm)$)"
            + r"|(?P<link>\[(http|\.\.\/|\.\/)(.*)\])"
            + r"|(?P<img>[^\s'\"]+\.(jpg|jpeg|png|gif)$)"
            + r"|(?P<audiostart>[^\s'\"]+\.(ogg|mp3)(\s*)autostart$)"
            + r"|(?P<audio>[^\s'\"]+\.(ogg|mp3)$)"
            + r"|(?P<url>(http|ftp|nntp|news|mailto|https)\:[^\s'\"]+\S)"
            + r"|(?P<ialink>\{(.*)\@(.*)\})"
            + r"|(?P<email>[-\w._+]+\@[\w.-]+)"
            + r"|(?P<li>^\s+\*(.*))"
            + r"|(?P<nothandled>nothandled)"
            + r"|(?P<pre>(\{\{\{|\}\}\}))"
            + r"|(?P<hidden_block>(\[\[(.*?)\:|\]\]))"
            + r")", re.IGNORECASE)
        blank_re = re.compile("^\s*$")
        indent_re = re.compile("^\s*")
        eol_re = re.compile(r'\r?\n')
        raw = string.expandtabs(self.raw)
        html_feed = u'<br/>\n'

        # fix some elements
        fix_element = re.sub(r"\[\[(.*?):", r"[[\1:\n", raw)
        raw = fix_element

        fix_element = re.sub(r"(.*?)\]\]", r"\1\n]]", raw)
        raw = fix_element

        # loop on lines
        for line in eol_re.split(raw):
            if not self.in_pre:
                if blank_re.match(line):
                    self.final_str += html_feed
                    continue
                indent = indent_re.match(line)
                self.final_str += self._indent_to(len(indent.group(0)))
            test = re.sub(scan_re, self.replace, line)
            self.final_str += test
            if self.in_li:
                test = re.sub(scan_re, self.replace, line[line.find("*")+1:].strip())
                self.final_str += test
                self.final_str += "</li>"
                self.in_li = 0
                
        if self.in_pre: self.final_str += u'</pre>\n'
        while len(self.hidden_block):
            self.hidden_block.pop(0)
            self.final_str += u'</div>\n'
        self.final_str += self._undent()
        if self.final_str == html_feed:
            return ""
        else:
            return self.final_str



#!/usr/bin/python
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# @author : pascal.fautrero@ac-versailles.fr

import re, math

class CurrentTransformation:
    """ Used to analyze transformation applied on svg element
        CTM (Current Transformation Matrix)
        expected forms :
            matrix(a b c d e f)
            translate(x y)
            translate(a)
            scale(a)
            scale(sx sy)
            rotate(angle cx cy)
            skewX(x)
            skewY(y)
   
    """

    def __init__(self):
        """Init"""
        self.matrix = [[1.0, 0.0, 0.0], [0.0, 1.0, 0.0]]
        self.rotate = 0
        self.rX = 0
        self.rY = 0
        self.translateX = 0
        self.translateY = 0
        self.scaleX = 1
        self.scaleY = 1

    def analyze(self,entry):
        """Analyze transform attribute"""
        regex_group = r'([^\s,]*)\s*,?\s*'
        
        # look for something with such a form : "matrix(a b c d e f)"
        regex_matrix = r'matrix\(\s*' + regex_group * 6 + r'\)'
        matchObj = re.match( regex_matrix, entry)
        if matchObj:
            self.extractMatrix(matchObj.groups())
            return

        # look for something with such a form : "translate(a)"
        regex_translate = r'translate\(\s*' + regex_group * 1 + r'\)'
        matchObj = re.match( regex_translate, entry)
        if matchObj:
            self.extractTranslate(matchObj.groups())
            return

        # look for something with such a form : "translate(a b)"
        regex_translate = r'translate\(\s*' + regex_group * 2 + r'\)'
        matchObj = re.match( regex_translate, entry)
        if matchObj:
            self.extractTranslate(matchObj.groups())
            return

        # look for something with such a form : "rotate(a)"
        regex_rotate = r'rotate\(\s*' + regex_group * 1 + r'\)'
        matchObj = re.match( regex_rotate, entry)
        if matchObj:
            self.extractRotate(matchObj.groups())
            return        
       
        # look for something with such a form : "rotate(a b c)"
        regex_rotate = r'rotate\(\s*' + regex_group * 3 + r'\)'
        matchObj = re.match( regex_rotate, entry)
        if matchObj:
            self.extractRotate(matchObj.groups())
            return        

        # look for something with such a form : "scale(a)"
        regex_scale = r'scale\(\s*' + regex_group * 1 + r'\)'
        matchObj = re.match( regex_scale, entry)
        if matchObj:
            self.extractScale(matchObj.groups())
            return        

        # look for something with such a form : "scale(a b)"
        regex_scale = r'scale\(\s*' + regex_group * 2 + r'\)'
        matchObj = re.match( regex_scale, entry)
        if matchObj:
            self.extractScale(matchObj.groups())
            return        

        
    def extractTranslate(self,groups):
        """extract a and b from translate(a b) pattern"""
        self.translateX = groups[0]
        self.translateY = groups[0]
        if len(groups) == 2 and groups[1]:
            self.translateY = groups[1]
        self.matrix = [[1.0, 0.0, float(self.translateX)], \
          [0.0, 1.0, float(self.translateY)]]

    def extractScale(self,groups):
        """extract a and b from scale(a b) pattern"""
        self.scaleX = groups[0]
        self.scaleY = groups[0]        
        if len(groups) == 2 and groups[1]:
            self.scaleY = groups[1]
        self.matrix = [[float(self.scaleX), 0.0, 0.0], \
          [0.0, float(self.scaleY), 0.0]]
        
        
    def extractRotate(self,groups):
        """extract a,b and c from rotate(a b c) pattern
            cos(a)  -sin(a)  -cx.cos(a) + cy.sin(a) + cx
            sin(a)   cos(a)  -cx.sin(a) - cy.cos(a) + cy
            0       0        1
        """
        self.rotate = groups[0]
        self.rX = "0"
        self.rY = "0"
        if len(groups) == 3:
            if groups[1]:
            	self.rX = groups[1]
            if groups[2]:
            	self.rY = groups[2]
            
        alpha = float(self.rotate)
        cx = float(self.rX)
        cy = float(self.rY)
        self.matrix = [ [
                          math.cos(alpha), 
                          -math.sin(alpha), 
                          -cx * math.cos(alpha) + cy * math.sin(alpha) + cx
                        ], 
                        [
                          math.sin(alpha), 
                          math.cos(alpha), 
                          -cx * math.sin(alpha) - cy * math.cos(alpha) + cy
                        ]
                    ]
        
    def extractMatrix(self,groups):
        """extract a,b,c,d,e,f from matrix(a b c d e f) pattern"""
        a = groups[0]
        b = groups[1]
        c = groups[2]
        d = groups[3]
        e = groups[4]
        f = groups[5]
        self.matrix=[[float(a),float(c),float(e)], [float(b),float(d),float(f)]]
        self.translateX = e
        self.translateY = f
        self.scaleX = math.sqrt(float(a)**2+float(c)**2)
        self.scaleY = math.sqrt(float(b)**2+float(d)**2)
        self.rotate = math.atan2(float(b),float(d))
        
    def rectToPath(self,node):
        """inspired from inkscape pathmodifier.py"""
        x = float(node['x'])
        y = float(node['y'])
        w = float(node['width'])
        h = float(node['height'])        
        rx = 0
        ry = 0
        if 'rx' in node:
            rx = float(node['rx'])
        if 'ry' in node:
            ry = float(node['ry'])

        if rx==0 or ry ==0:
            d ='M %f,%f '%(x,y)
            d+='L %f,%f '%(x+w,y)
            d+='L %f,%f '%(x+w,y+h)
            d+='L %f,%f '%(x,y+h)
            d+='L %f,%f '%(x,y)
        else:
            d ='M %f,%f '%(x+rx,y)
            d+='L %f,%f '%(x+w-rx,y)
            d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,x+w,y+ry)
            d+='L %f,%f '%(x+w,y+h-ry)
            d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,x+w-rx,y+h)
            d+='L %f,%f '%(x+rx,y+h)
            d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,x,y+h-ry)
            d+='L %f,%f '%(x,y+ry)
            d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,x+rx,y)
            
        return d

    def circleToPath(self,node):
        """inspired from inkscape pathmodifier.py"""
        cx = float(node['cx'])
        cy = float(node['cy'])
        r = 0
        if 'r' in node:
            r = float(node['r'])

        d ='M %f,%f '%(cx-r,cy)
        d+='A %f,%f 0 0 1 %f,%f'%(r,r,cx,cy-r)
        d+='A %f,%f 0 0 1 %f,%f'%(r,r,cx+r,cy)
        d+='A %f,%f 0 0 1 %f,%f'%(r,r,cx,cy+r)
        d+='A %f,%f 0 0 1 %f,%f'%(r,r,cx-r,cy)

        return d

    def ellipseToPath(self,node):
        """inspired from inkscape pathmodifier.py"""
        cx = float(node['cx'])
        cy = float(node['cy'])
        rx = 0
        ry = 0
        if 'rx' in node:
            rx = float(node['rx'])
        if 'ry' in node:
            ry = float(node['ry'])

        d ='M %f,%f '%(cx-rx,cy)
        d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,cx,cy-ry)
        d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,cx+rx,cy)
        d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,cx,cy+ry)
        d+='A %f,%f 0 0 1 %f,%f'%(rx,ry,cx-rx,cy)

        return d

    def applyTransformToPoint(self,mat,pt):
        x = mat[0][0]*pt[0] + mat[0][1]*pt[1] + mat[0][2]
        y = mat[1][0]*pt[0] + mat[1][1]*pt[1] + mat[1][2]
        pt[0]=x
        pt[1]=y

    def applyTransformToPath(self,mat,path):
        for comp in path:
            for ctl in comp:
                for pt in ctl:
                    self.applyTransformToPoint(mat,pt)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals 
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import NoAlertPresentException
from selenium.webdriver.common.action_chains import ActionChains
import unittest, time, re
import os

if os.name != 'nt':
    from pyvirtualdisplay import Display
    display = Display(visible=0, size=(1024, 768))
    display.start()

class Test(unittest.TestCase):
    def setUp(self):
        #chromedriver = "/usr/bin/google-chrome"
        #os.environ["webdriver.chrome.driver"] = chromedriver
        self.driver = webdriver.Chrome()
        self.driver.implicitly_wait(30)
        self.base_url = "file://"+os.path.dirname(os.path.abspath(__file__))+"/index.html"
        self.verificationErrors = []
        self.accept_next_alert = True
    
    def test_init(self):
        driver = self.driver
        driver.get(self.base_url)
        self.assertEqual("Titre du document", driver.find_element_by_id("title").text)
        self.assertTrue(driver.find_element_by_id("title").is_displayed())
        self.assertEqual("Titre de l'image", driver.find_element_by_id("collapsecomment-heading").text)
        self.assertTrue(driver.find_element_by_id("collapsecomment-heading").is_displayed())
        self.assertEqual("Titre du rectangle ***gras*** **italique**", driver.find_element_by_id("collapse0-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse0-heading").is_displayed())
        self.assertEqual("Titre de l'ellipse", driver.find_element_by_id("collapse1-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse1-heading").is_displayed())
        self.assertEqual(u"Titre de l'étoile {{{Texte brut}}}", driver.find_element_by_id("collapse2-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse2-heading").is_displayed())
        self.assertEqual("Titre de ligne", driver.find_element_by_id("collapse3-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse3-heading").is_displayed())
        self.assertEqual("Titre bezier", driver.find_element_by_id("collapse4-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse4-heading").is_displayed())
        self.assertEqual("son 2", driver.find_element_by_id("collapse5-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse5-heading").is_displayed())
        self.assertEqual("Son 1", driver.find_element_by_id("collapse6-heading").text)
        self.assertTrue(driver.find_element_by_id("collapse6-heading").is_displayed())
        self.assertEqual("""Description de l'image<video controls="" preload="none" data-state="none">
	            <source type="video/mp4" src="../media-share/1.mp4">
	            <source type="video/ogg" src="../media-share/1.ogv">
	            <source type="video/webm" src="../media-share/1.webm">
            </video>

        """, driver.find_element_by_xpath("//div[@id='collapsecomment']/div").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "//div[@id='collapsecomment']/div/video"))
        assert """Description du rectangle <b>gras</b> <em>italique</em><div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#" data-password=""" in driver.find_element_by_xpath("//div[@id='collapse0']/div").get_attribute('innerHTML')
        assert """Voici la vidéo :<video controls="" preload="none" data-state="autostart">
	            <source type="video/mp4" src="../media-share/1.mp4">
	            <source type="video/ogg" src="../media-share/1.ogv">
	            <source type="video/webm" src="../media-share/1.webm">
            </video>""" in driver.find_element_by_xpath("//div[@id='collapse0']/div").get_attribute('innerHTML')
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse0')/div/video"))
        self.assertEqual("""
          <div class="accordion-inner">Description de l'ellipse<ul>
<li>une liste  <a href="http://dane.ac-versailles.fr" target="_blank">Le site de la Dane</a></li><li>de puces</li><ul>
<li>sur 2</li><li>niveaux</li><br>
</ul>
</ul>
Quelle est la bonne réponse ?<img src="../media-share/1.jpg">

          </div>
      """, driver.find_element_by_xpath("//div[@id='collapse1']").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse1')/div/img"))
        self.assertEqual("""
          <div class="accordion-inner">Description de l'étoile<ul>
 <a href="http://dane.ac-versailles.fr" target="_blank">Le site de la Dane</a></ul>
<pre>Texte brut</pre>
<br>
<img src="../media-share/1.gif">

          </div>
      """, driver.find_element_by_xpath("//div[@id='collapse2']").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse2')/div/img"))
        self.assertEqual("""
          <div class="accordion-inner">Description de ligne1<br>
<video controls="" preload="none" data-state="autostart">
	            <source type="video/mp4" src="../media-share/1.mp4">
	            <source type="video/ogg" src="../media-share/1.ogv">
	            <source type="video/webm" src="../media-share/1.webm">
            </video>
<br>

          </div>
      """, driver.find_element_by_xpath("//div[@id='collapse3']").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse3')/div/video"))
        self.assertEqual("""
          <div class="accordion-inner">Description de beziertracer
<hr>
une ligne<br>
<img src="../media-share/1.png">

          </div>
      """, driver.find_element_by_xpath("//div[@id='collapse4']").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse4')/div/img"))
        assert """<div class="accordion-inner">le son 2 ! <audio controls="" data-state="autostart">
	            <source type="audio/ogg" src="../media-share/1.ogg">
	            <source type="audio/mp3" src="../media-share/1.mp3">
            </audio>
<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#" data-target=""" in driver.find_element_by_xpath("//div[@id='collapse5']").get_attribute('innerHTML')
        assert """La réponse 2</ul>
</div>
LA réponse à la question<br>

          </div>
      """ in driver.find_element_by_xpath("//div[@id='collapse5']").get_attribute('innerHTML')
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse5')/div/audio"))
        self.assertEqual("""
          <div class="accordion-inner">le son 1 !<audio controls="" data-state="none">
	            <source type="audio/ogg" src="../media-share/1.ogg">
	            <source type="audio/mp3" src="../media-share/1.mp3">
            </audio>

          </div>
      """, driver.find_element_by_xpath("//div[@id='collapse6']").get_attribute('innerHTML'))
        self.assertTrue(self.is_element_present(By.XPATH, "id('collapse6')/div/audio"))
        self.check_element()
        webdriver.common.action_chains.ActionChains(driver).move_to_element_with_offset(driver.find_element_by_css_selector("a.infos"), 5, 5).click().perform()
#        time.sleep(2)
        self.assertEqual("Michaël Nourry <br>", driver.find_element_by_xpath("//article[@id='popup_text']/p").get_attribute('innerHTML'))
#        self.check_element("id('popup')")
#        time.sleep(3)
           
    def test_nav_1(self):
        driver = self.driver
        driver.get(self.base_url)
        time.sleep(2)
        webdriver.common.action_chains.ActionChains(driver).move_to_element_with_offset(driver.find_element_by_id("collapsecomment-heading"), 5, 5).click().perform()
#        time.sleep(2)
#        self.check_element("collapsecomment")
#        time.sleep(5)
        webdriver.common.action_chains.ActionChains(driver).move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 5, 5).click().perform()
#        self.check_element("collapse0")
#        driver.execute_script(self.click_zone("Son 1"))
#        driver.execute_script(self.click_zone("Son 1"))
#        time.sleep(5)
        self.assertTrue(driver.find_element_by_xpath("id('collapse3-heading')").is_displayed())
#        driver.execute_script(self.click_zone("Son 1"))
        self.assertTrue(driver.find_element_by_xpath("id('collapse3-heading')").is_displayed())
        """
        time.sleep(5)
        self.check_element("collapse3")
        driver.find_element_by_id("collapse5-heading").click()
        self.check_element("collapse5")
        driver.find_element_by_id("collapse6-heading").click()
        self.check_element("collapse6")
        """




    def test_nav_2(self):
        driver = self.driver
        driver.get(self.base_url)
        time.sleep(5)
        action = webdriver.common.action_chains.ActionChains(driver)
        action.move_to_element_with_offset(driver.find_element_by_css_selector("a.infos"), 5, 5).click().perform()
#        time.sleep(5)
#        self.check_element("popup")
#       time.sleep(10)
#        driver.find_element_by_id("popup_close").click()
#        self.check_element()
        time.sleep(4)
#        driver.execute_script(self.click_zone("son 2"))
#        self.check_element("collapse5")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 1, 1).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 2, 2).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"),3, 3).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 4,4).click().perform()
        action.move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 5, 5).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 6, 6).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"),7, 7).click().move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 8,8).click().perform()
#        self.check_element("collapse0")
        """action.move_to_element_with_offset(driver.find_element_by_id("collapse1-heading"), 5, 5).click().perform()
        self.check_element("collapse1")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse2-heading"), 5, 5).click().perform()
        self.check_element("collapse2")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse4-heading"), 5, 5).click().perform()
        self.check_element("collapse4")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse5-heading"), 5, 5).click().perform()
        self.check_element("collapse5")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse6-heading"), 5, 5).click().perform()
        self.check_element("collapse6")
        action.move_to_element_with_offset(driver.find_element_by_id("collapsecomment-heading"), 5, 5).click().perform()
        self.check_element("collapsecomment")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 5, 5).click().perform()
        self.check_element("collapse0")
        self.assertEqual("Description du rectangle gras italiqueRéponse:Voici la vidéo :", driver.find_element_by_css_selector("#collapse0 > div.accordion-inner").text)
        action.move_to_element_with_offset(driver.find_element_by_id("collapsecomment-heading"), 5, 5).click().perform()
        self.check_element("collapsecomment")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse0-heading"), 5, 5).click().perform()
        self.check_element("collapse0")
        action.move_to_element_with_offset(driver.find_element_by_id("collapse4-heading"), 5, 5).click().perform()
        self.check_element("collapse4")
        """
    def test_init_1(self):
        driver = self.driver
        self.base_url = "file://"+os.path.dirname(os.path.abspath(__file__))+"/1.html"
        self.test_init()

    def test_nav_1_1(self):
        driver = self.driver
        self.base_url = "file://"+os.path.dirname(os.path.abspath(__file__))+"/1.html"
        self.test_nav_1()

    def test_nav_2_1(self):
        driver = self.driver
        self.base_url = "file://"+os.path.dirname(os.path.abspath(__file__))+"/1.html"
        self.test_nav_2()
        
    def is_element_present(self, how, what):
        try: self.driver.find_element(by=how, value=what)
        except NoSuchElementException, e: return False
        return True
    
    def is_alert_present(self):
        try: self.driver.switch_to_alert()
        except NoAlertPresentException, e: return False
        return True
    
    def close_alert_and_get_its_text(self):
        try:
            alert = self.driver.switch_to_alert()
            alert_text = alert.text
            if self.accept_next_alert:
                alert.accept()
            else:
                alert.dismiss()
            return alert_text
        finally: self.accept_next_alert = True

    def click_zone(self,zone):
        return """
		    for (var i in kinetic.shapes) {
			    if (kinetic.shapes[i].attrs['name'] == '"""+zone+"""') {
				    kinetic.shapes[i].fire('click');
                    break;
			    }
		    }
           """
    def waitAndSee(self,true_or_not):
        for t in range(30):
            time.sleep(1)
            print(t)
            if(true_or_not.is_displayed()):
                break

    def check_element(self,visible="0XXXX0XXXX0"):
        time.sleep(1)
        driver = self.driver
        elements = [
            "id('popup')",
            "id('collapsecomment')/div",
            "//div[@id='collapsecomment']/div/video",
            "id('collapse0')/div",
            "id('collapse0')/div/video",
            "id('collapse1')/div/ul",
            "id('collapse1')/div/img",
            "id('collapse2')/div",
            "id('collapse2')/div/img",
            "id('collapse3')/div",
            "id('collapse3')/div/video",
            "id('collapse4')/div",
            "id('collapse4')/div/img",
            "id('collapse5')/div",
            "id('collapse5')/div/audio",
            "id('collapse6')/div",
            "id('collapse6')/div/audio"
            ]
        for id in elements:
            if id.find(visible)+1:
                self.assertTrue(driver.find_element_by_xpath(id).is_displayed())
            else:
                self.assertFalse(driver.find_element_by_xpath(id).is_displayed())

    def tearDown(self):
        self.driver.close()
#        self.assertEqual([], self.verificationErrors)
#        display.stop()

if __name__ == "__main__":
    unittest.main()
 



#!/usr/bin/python
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# @author : pascal.fautrero@ac-versailles.fr

from xiaconverter.pikipiki import PageFormatter
from nose.tools import *
import re

class TestPageFormatter:

    def test_print_html(self):
        raw = "**text**";
        expected_output = "<em>text</em>";
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html2(self):
        raw = "***text***";
        expected_output = "<b>text</b>";
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html3(self):
        raw = "{{{text}}}";
        expected_output = "<pre>\ntext</pre>\n";
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html4(self):
        raw = "photo.jpg";
        expected_output = '<img src="photo.jpg"/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)        

    def test_print_html5(self):
        raw = "photo.jpeg";
        expected_output = '<img src="photo.jpeg"/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output) 

    def test_print_html6(self):
        raw = "photo.png";
        expected_output = '<img src="photo.png"/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output) 

    def test_print_html7(self):
        raw = "photo.gif";
        expected_output = '<img src="photo.gif"/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output) 

    def test_print_html8(self):
        raw = "admin@test.com";
        expected_output = '<a href="mailto:admin@test.com">admin@test.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html9(self):
        raw = "<&>";
        expected_output = '&lt;&amp;&gt;';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html10(self):
        raw = "----";
        expected_output = '\n<hr/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html11(self):
        raw = "-----";
        expected_output = '\n<hr size=3/>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html12(self):
        raw = '&lt;iframe src=&quot;http://example.com&quot;&gt;&lt;/iframe&gt;';
        expected_output = '<div class="videoWrapper4_3" data-iframe="http://example.com"></div>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html13(self):
        raw = '&lt;iframe width="560" height="315" src=&quot;//example.com&quot;&gt;&lt;/iframe&gt;';
        expected_output = '<div class="videoWrapper16_9" data-iframe="http://example.com"></div>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output) 

    def test_print_html14(self):
        raw = '<iframe width="560" height="315" src="//www.youtube.com/embed/ctXwWpMz44M" frameborder="0" allowfullscreen></iframe>';
        expected_output = '<div class="videoWrapper16_9" data-iframe="http://www.youtube.com/embed/ctXwWpMz44M"></div>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output) 

    def test_print_html15(self):
        raw = "http://example.com";
        expected_output = '<a href="http://example.com" target="_blank">http://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)  

    def test_print_html16(self):
        raw = "https://example.com";
        expected_output = '<a href="https://example.com" target="_blank">https://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html17(self):
        raw = "ftp://example.com";
        expected_output = '<a href="ftp://example.com" target="_blank">ftp://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html18(self):
        raw = "nntp://example.com";
        expected_output = '<a href="nntp://example.com" target="_blank">nntp://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html19(self):
        raw = "news://example.com";
        expected_output = '<a href="news://example.com" target="_blank">news://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html20(self):
        raw = "mailto://example.com";
        expected_output = '<a href="mailto://example.com" target="_blank">mailto://example.com</a>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)         

    def test_print_html21(self):
        raw = " * line 1\n * line2";
        expected_output = '<ul>\n<li>line 1</li><li>line2</li></ul>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html22(self):
        raw = " * ***line 1***\n * [http://test line2]";
        expected_output = '<ul>\n<li><b>line 1</b></li><li><a href="http://test" target="_blank">line2</a></li></ul>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html23(self):
        raw = "   * line 1\n  * line2";
        expected_output = '<ul>\n<li>line 1</li></ul>\n<ul>\n<li>line2</li></ul>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html24(self):
        raw = "video.mp4";
        expected_output = '<video controls preload="none" data-state="none">\n\t            <source type="video/mp4" src="video.mp4" />\n\t            <source type="video/ogg" src="video.ogv" />\n\t            <source type="video/webm" src="video.webm" />\n            </video>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html25(self):
        raw = "video.mp4 autostart";
        expected_output = '<video controls preload="none" data-state="autostart">\n\t            <source type="video/mp4" src="video.mp4" />\n\t            <source type="video/ogg" src="video.ogv" />\n\t            <source type="video/webm" src="video.webm" />\n            </video>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html26(self):
        raw = "video.ogv";
        expected_output = '<video controls preload="none" data-state="none">\n\t            <source type="video/mp4" src="video.mp4" />\n\t            <source type="video/ogg" src="video.ogv" />\n\t            <source type="video/webm" src="video.webm" />\n            </video>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html27(self):
        raw = "video.webm";
        expected_output = '<video controls preload="none" data-state="none">\n\t            <source type="video/mp4" src="video.mp4" />\n\t            <source type="video/ogg" src="video.ogv" />\n\t            <source type="video/webm" src="video.webm" />\n            </video>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html28(self):
        raw = "audio.mp3";
        expected_output = '<audio controls data-state="none">\n\t            <source type="audio/ogg" src="audio.ogg" />\n\t            <source type="audio/mp3" src="audio.mp3" />\n            </audio>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html29(self):
        raw = "audio.ogg";
        expected_output = '<audio controls data-state="none">\n\t            <source type="audio/ogg" src="audio.ogg" />\n\t            <source type="audio/mp3" src="audio.mp3" />\n            </audio>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html30(self):
        raw = "audio.mp3  autostart";
        expected_output = '<audio controls data-state="autostart">\n\t            <source type="audio/ogg" src="audio.ogg" />\n\t            <source type="audio/mp3" src="audio.mp3" />\n            </audio>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html31(self):
        raw = "audio.ogg    autostart";
        expected_output = '<audio controls data-state="autostart">\n\t            <source type="audio/ogg" src="audio.ogg" />\n\t            <source type="audio/mp3" src="audio.mp3" />\n            </audio>\n';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html32(self):
        raw = "[http://example.com A small test]";
        expected_output = '<a href="http://example.com" target="_blank">A small test</a>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html33(self):
        raw = "[http://example.com]";
        expected_output = '<a href="http://example.com" target="_blank">http://example.com</a>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html34(self):
        raw = "[../index.html test]";
        expected_output = '<a href="../index.html" target="_blank">test</a>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html35(self):
        raw = "[./index.html test]";
        expected_output = '<a href="./index.html" target="_blank">test</a>';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html36(self):
        raw = " ";
        expected_output = '';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html37(self):
        raw = "}}}";
        expected_output = '';
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)

    def test_print_html38(self):
        raw = "[[answer : my answer]]";
        expected_output = '<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#"  data-target="##">answer </a></div><div class="response" id="response_##"><ul>\n my answer</ul>\n</div>\n';
        output = PageFormatter(raw).print_html()
        target_id = ""
        target_entry = re.search('data-target="(.*?)"', output, re.IGNORECASE|re.DOTALL)
        if target_entry:
            target_id = target_entry.group(1)
            output = output.replace(target_id, "##")
        assert_equal(expected_output, output)

    def test_print_html39(self):
        raw = "[[answer (code=123456):\nmy answer]]";
        expected_output = '<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b" data-target="##">answer </a></div><form class="unlock" style="display:none;" id="form_##"><input type="text"><input type="submit" data-target="##" value="" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b"></form><div class="response" id="response_##">DVBBGws8XEsTVVtFRldB</div>\n';
        output = PageFormatter(raw).print_html()
        target_id = ""
        target_entry = re.search('data-target="(.*?)"', output, re.IGNORECASE|re.DOTALL)
        if target_entry:
            target_id = target_entry.group(1)
            output = output.replace(target_id, "##")
        assert_equal(expected_output, output)

    def test_print_html40(self):
        raw = "[[answer (code=123456):\nmy answer\n]]";
        expected_output = '<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b" data-target="##">answer </a></div><form class="unlock" style="display:none;" id="form_##"><input type="text"><input type="submit" data-target="##" value="" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b"></form><div class="response" id="response_##">DVBBGws8XEsTVVtFRldBCFdEHgw5</div>\n';
        output = PageFormatter(raw).print_html()
        target_id = ""
        target_entry = re.search('data-target="(.*?)"', output, re.IGNORECASE|re.DOTALL)
        if target_entry:
            target_id = target_entry.group(1)
            output = output.replace(target_id, "##")
        assert_equal(expected_output, output)

    def test_print_html41(self):
        raw = "[[answer (code=123456):my answer]]";
        expected_output = '<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b" data-target="##">answer </a></div><form class="unlock" style="display:none;" id="form_##"><input type="text"><input type="submit" data-target="##" value="" data-password="7c4a8d09ca3762af61e59520943dc26494f8941b"></form><div class="response" id="response_##">XEsTVVtFRldB</div>\n';
        output = PageFormatter(raw).print_html()
        target_id = ""
        target_entry = re.search('data-target="(.*?)"', output, re.IGNORECASE|re.DOTALL)
        if target_entry:
            target_id = target_entry.group(1)
            output = output.replace(target_id, "##")
        assert_equal(expected_output, output)

    def test_print_html42(self):
        raw = "[[answer : my answer";
        expected_output = '<div style="margin-top:5px;margin-bottom:5px;"><a class="button" href="#"  data-target="##">answer </a></div><div class="response" id="response_##"><ul>\n my answer</div>\n</ul>';
        output = PageFormatter(raw).print_html()
        target_id = ""
        target_entry = re.search('data-target="(.*?)"', output, re.IGNORECASE|re.DOTALL)
        if target_entry:
            target_id = target_entry.group(1)
            output = output.replace(target_id, "##")
        assert_equal(expected_output, output)

    def test_print_html43(self):
        raw = "nothandled";
        expected_output = "Can't handle match nothandled";
        output = PageFormatter(raw).print_html()
        assert_equal(expected_output, output)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright: 2014 Francois Lafont <francois.lafont@ac-versailles.fr>
#
# License: GPL-3.0+
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import subprocess
from distutils.core import setup
from distutils.cmd import Command
from distutils.command.install import install
from distutils.command.clean import clean

## Important paths and filenames.
setup_dir = os.path.dirname(os.path.abspath(__file__))
build_dir = os.path.join(setup_dir, 'build')
make_dir = os.path.join(setup_dir, 'make')
src_dir = os.path.join(setup_dir, 'src')
changelog = os.path.join(setup_dir, 'CHANGELOG.md')
readme = os.path.join(setup_dir, 'README.md')


def xia_build():
    # Recreate a Cleaned build directory.
    if os.path.isdir(build_dir):
        shutil.rmtree(build_dir)
    shutil.copytree(src_dir, build_dir)

    # Generate .mo files.
    subprocess.check_call([os.path.join(make_dir, "generate_mo.sh")])

    # Build js in each theme and in vendors/.
    subprocess.check_call([os.path.join(make_dir, "build_js.sh")])


def update_po():
    subprocess.check_call([os.path.join(make_dir, "update_po.sh")])


class UpdatePO(Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""
        pass

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""
        pass

    def run(self):
        update_po()


class BuildStandalone(Command):
    user_options = []

    def initialize_options(self):
        """Abstract method that is required to be overwritten"""
        pass

    def finalize_options(self):
        """Abstract method that is required to be overwritten"""
        pass

    def run(self):
        xia_build()


class Install(install):

    def run(self):
        install.run(self)
        xia_build()

class Clean(clean):

    def run(self):
        clean.run(self)
        if os.path.isdir(build_dir):
            shutil.rmtree(build_dir)


# Get the version of the application.
with open(changelog, 'r') as f:
  line1 = f.readline()
words = line1.split()
version = words[1]

# Get the long description of the application.
with open(readme, 'r') as f:
    long_description = f.read()

setup(
    name='xia',
    version=version,
    packages=['xiaconverter'],
    package_dir={ '': 'src'},
    cmdclass={
        'buildstandalone': BuildStandalone,
        'install': Install,
        'clean': Clean,
        'update_po': UpdatePO,
    },
    author='Pascal Fautrero',
    author_email='pascal.fautrero@ac-versailles.fr',
    description='Convert svg to full html5 interactive pictures',
    long_description=long_description,
    url='http://xia.dane.ac-versailles.fr',
    license='GPL-3',
)





#!/usr/bin/python
# -*- coding: utf-8 -*-
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>
#   
# @author : pascal.fautrero@crdp.ac-versailles.fr

import gettext
import locale

class hook:
    """do some stuff during image active generations"""

    def __init__(self, root, iaobject, PageFormatter, langPath):
        """Init"""
        try:
            t = gettext.translation("xia-converter", langPath, languages=[locale.getdefaultlocale()[0]])
        except:
            t = gettext.translation("xia-converter", langPath, languages=['en_US'])
        translate = t.ugettext
        self.root = root
        self.iaobject = iaobject
        self.PageFormatter = PageFormatter
        self.tooltip = translate("export audioBrown")
        self.loading = translate("loading")

    def generateIndex(self,filePath, templatePath):
        """ generate index file"""
        
        final_str  = u'<article class="detail_content" id="general">\n'
        final_str += '<img class="article_close" src="{{LogoClose}}" alt="close"/>'
        final_str += u'  <h1>' + self.iaobject.scene["intro_title"] + '</h1>\n'
        final_str += u'  <p>' + self.PageFormatter(self.iaobject.scene["intro_detail"]).print_html() + u'</p>\n'
        final_str += u'</article>\n'
        for i, detail in enumerate(self.iaobject.details):
            if detail['options'].find(u"direct-link") == -1:
                dataState = "full"
                if self.PageFormatter(detail["detail"]).print_html() == "":
                    dataState = "void"
                final_str += u'<article class="detail_content" data-state="' + dataState + '" id="article-'+unicode(str(i), "utf8") + u'">\n'
                final_str += '<img class="article_close" src="{{LogoClose}}" alt="close"/>'
                if detail['title'] != "":
                    final_str += u'  <h1>' + detail['title'] + u'</h1>\n'
                final_str += u'  <div>' + self.PageFormatter(detail["detail"]).print_html() + u'</div>\n'
                final_str += u'</article>\n'

        with open(templatePath,"r") as template:
            final_index = template.read().decode("utf-8")

            metadatas = ""
            if self.iaobject.scene["creator"]:
                metadatas += self.iaobject.scene["creator"] + "<br/>"
            if self.iaobject.scene["rights"]:
                metadatas += self.iaobject.scene["rights"] + "<br/>"
            if self.iaobject.scene["publisher"]:
                metadatas += self.iaobject.scene["publisher"] + "<br/>"
            if self.iaobject.scene["identifier"]:
                metadatas += self.iaobject.scene["identifier"] + "<br/>"
            if self.iaobject.scene["coverage"]:
                metadatas += self.iaobject.scene["coverage"] + "<br/>"
            if self.iaobject.scene["source"]:
                metadatas += self.iaobject.scene["source"] + "<br/>"
            if self.iaobject.scene["relation"]:
                metadatas += self.iaobject.scene["relation"] + "<br/>"
            if self.iaobject.scene["language"]:
                metadatas += self.iaobject.scene["language"] + "<br/>"
            if self.iaobject.scene["contributor"]:
                metadatas += self.iaobject.scene["contributor"] + "<br/>"
            if self.iaobject.scene["date"]:
                metadatas += self.iaobject.scene["date"] + "<br/>"

            final_index = final_index.replace("{{METADATAS}}", metadatas)
            final_index = final_index.replace("{{AUTHOR}}", self.iaobject.scene["creator"])
            final_index = final_index.replace("{{DESCRIPTION}}", self.iaobject.scene["description"])
            final_index = final_index.replace("{{KEYWORDS}}", self.iaobject.scene["keywords"])
            final_index = final_index.replace("{{TITLE}}", self.iaobject.scene["title"])
            final_index = final_index.replace("{{CONTENT}}", final_str)
            final_index = final_index.replace("{{LOADING}}", self.loading)
            if self.root.index_standalone:
                xiaWebsite = "http://xia.dane.ac-versailles.fr/network/delivery/audioBrown"
                final_index = final_index.replace("{{MainCSS}}", xiaWebsite + "/css/main.css")
                final_index = final_index.replace("{{LogoLoading}}",  xiaWebsite + "/img/xia.png")
                final_index = final_index.replace("{{LogoClose}}", xiaWebsite + "/img/close.png")
                final_index = final_index.replace("{{datasJS}}", "<script>" + self.iaobject.jsonContent + "</script>")
                final_index = final_index.replace("{{lazyDatasJS}}", '')
                final_index = final_index.replace("{{JqueryJS}}", "https://code.jquery.com/jquery-1.11.1.min.js")
                final_index = final_index.replace("{{sha1JS}}", xiaWebsite + "/js/git-sha1.min.js")
                final_index = final_index.replace("{{kineticJS}}", "https://cdn.jsdelivr.net/kineticjs/5.1.0/kinetic.min.js")
                final_index = final_index.replace("{{xiaJS}}", xiaWebsite + "/js/xia.js")
                final_index = final_index.replace("{{hooksJS}}", xiaWebsite + "/js/hooks.js")
                final_index = final_index.replace("{{labJS}}", "https://cdnjs.cloudflare.com/ajax/libs/labjs/2.0.3/LAB.min.js")
            else:
                final_index = final_index.replace("{{MainCSS}}", "css/main.css")
                final_index = final_index.replace("{{LogoLoading}}",  "img/xia.png")
                final_index = final_index.replace("{{LogoClose}}", "img/close.png")
                final_index = final_index.replace("{{datasJS}}", "")
                final_index = final_index.replace("{{lazyDatasJS}}", '.script("datas/data.js")')
                final_index = final_index.replace("{{JqueryJS}}", "js/jquery.min.js")
                final_index = final_index.replace("{{sha1JS}}", "js/git-sha1.min.js")
                final_index = final_index.replace("{{kineticJS}}", "js/kinetic.min.js")
                final_index = final_index.replace("{{xiaJS}}", "js/xia.js")
                final_index = final_index.replace("{{hooksJS}}", "js/hooks.js")
                final_index = final_index.replace("{{labJS}}", "js/LAB.min.js")  
        with open(filePath,"w") as indexfile:
            indexfile.write(final_index.encode("utf-8"))
